  /*    by Enrico Rolfi */
  
  
  #pragma once
  #pragma pack(1)


  typedef struct _RESMAPDIRSCI {
          unsigned char  type;               // Type of resources, 0xFF - end of list
          unsigned short list;               // Offset of IDs list
  } RESMAPDIRSCI, *PRESMAPDIRSCI;
  #define RESMAPDIRSCISIZE (sizeof(_RESMAPDIRSCI))


  typedef struct _RESMAPIDSSCI32 {
          unsigned short id;                 // ID of resource
          unsigned long  offset;             // Offset of resource in volume
  } RESMAPIDSSCI32, *PRESMAPIDSSCI32;
  #define RESMAPIDSSCI32SIZE (sizeof(_RESMAPIDSSCI32))


  typedef struct _RESVOLENTRYSCI32 {
          unsigned char  type;               // Type of resource
          unsigned short id;                 // ID of resource
          unsigned long  paklen;             // Size of packed resource (stored in vol)
          unsigned long  unplen;             // Size of unpacked resource (actual size)
          unsigned short pakmeth;            // Packing method
  } RESVOLENTRYSCI32, *PRESVOLENTRYSCI32;
  #define RESVOLENTRYSCI32SIZE (sizeof(_RESVOLENTRYSCI32))
  
  

 
  struct PROGRAMENTRYSCI32 {
//RIDONDANTE  unsigned char  type;               // Type of resource
          unsigned short id;                 // ID of resource
          unsigned long  paklen;             // Size of packed resource (stored in vol)
          unsigned long  unplen;             // Size of unpacked resource (actual size)
          unsigned short pakmeth;            // Packing method
          char  file;
          unsigned long  offset;         //offset in resSCIfile
  };
    
  #pragma pack()
   
  class ResMap
  {
        List<PROGRAMENTRYSCI32>     *_sciDirs[32];  //meglio fare l'array cosi li posso ordinare per resID
        unsigned long               _versionStamp;
  
  public:
        ResMap() :_versionStamp(0) { for (int i=0; i<32; i++) _sciDirs[i] = new List<PROGRAMENTRYSCI32>; }
        ~ResMap() { delete [] _sciDirs; } 
         
        unsigned long VersionStamp() const { return _versionStamp; }
        void VersionStamp(unsigned long setto) { _versionStamp = setto; } 
        List<PROGRAMENTRYSCI32> *getDir(const unsigned char which) const;
        int addDir(unsigned char type, unsigned short items);
        unsigned short getDirSize(unsigned char dir) const { return _sciDirs[dir]->getSize(); }
        unsigned short getDirItemsCount(unsigned char dir) const { return _sciDirs[dir]->getLast(); }
        PROGRAMENTRYSCI32 *getFile(unsigned char dir, unsigned short id) const;
        PROGRAMENTRYSCI32 *getFileByNum(unsigned char dir, unsigned short num) const;
        int addFile(unsigned char dir, const PROGRAMENTRYSCI32 *entry, bool replace, bool sort);
        unsigned char getDirCount() const;

  };
  
  
  unsigned char ResMap::getDirCount() const
  {
    unsigned char tcount=0;
    for (int i=0; i<32; i++)
        if (getDirSize(i)>0)
           tcount++;
           
    return tcount;
  }
  
  List<PROGRAMENTRYSCI32> *ResMap::getDir(const unsigned char which) const
  {
     if (_sciDirs[which]->getSize() != 0)
        return _sciDirs[which];
     
     return 0;                       
  }
  
  int ResMap::addDir(unsigned char type, unsigned short items)
  {
      type &= 0x7F;           //nel caso ci siano 0x80, 0x81 ecc
//printf("type %X, size %X\n",type, _sciDirs[type]->getSize());
      if (_sciDirs[type]->getSize() == 0)
      {
         delete _sciDirs[type];
         _sciDirs[type] = new List<PROGRAMENTRYSCI32>(items);
      
         return 0;      //OK
      }
      
        
      return _sciDirs[type]->getSize();        //ERROR!    
  }
  
  PROGRAMENTRYSCI32 *ResMap::getFile(unsigned char dir, unsigned short id) const
  {
     dir &= 0x7F;     //nel caso ci siano 0x80, 0x81 ecc
     if (_sciDirs[dir]->getSize() != 0)
        for (unsigned short i=0; i<_sciDirs[dir]->getLast(); i++)
            if (_sciDirs[dir]->getItem(i)->id == id)
               return _sciDirs[dir]->getItem(i);
     
     return NULL;     //not found
  }
  
  PROGRAMENTRYSCI32 *ResMap::getFileByNum(unsigned char dir, unsigned short num) const
  {
     dir &= 0x7F;     //nel caso ci siano 0x80, 0x81 ecc
     if ((_sciDirs[dir]->getSize() != 0) && (num < _sciDirs[dir]->getLast()))
        return _sciDirs[dir]->getItem(num);
     
     return NULL;     //not found
  }
  
  int ResMap::addFile(unsigned char dir, const PROGRAMENTRYSCI32 *entry, bool replace, bool sort)
  {
     dir &= 0x7F;     //nel caso ci siano 0x80, 0x81 ecc
     if (_sciDirs[dir]->getSize() != 0)
     {
        PROGRAMENTRYSCI32 *oldone = getFile(dir, entry->id);
        if (!sort)
        { 
           if (replace && (oldone!=NULL))
           {
              //_sciDirs[dir]->changeItem(entry);
              //operazione delicata:
              //memcpy(oldone, entry, sizeof(PROGRAMENTRYSCI32));
              *oldone = *entry;
              
              return 0;                     //no error
        
           }
           else
           {
               if (oldone == NULL)  //se non  gi stato inserito
               {   
                   _sciDirs[dir]->addItem(entry);
                   return 0;                     //no error
               }              
               return 1;                        //file already exists
           }
        }
        else //sorted
        {
             if (oldone == NULL)  //se non  gi stato inserito
             {   
                 unsigned short pos=0;
                 PROGRAMENTRYSCI32 *curitem = getFileByNum(dir, pos);
                 if (curitem!=NULL)
                 {
                    PROGRAMENTRYSCI32 *lastitem = curitem;
                    if (curitem->id < entry->id)
                       for (pos=1; pos<_sciDirs[dir]->getLast(); pos++)
                       {
                           curitem = getFileByNum(dir, pos);
                           if ((curitem->id > entry->id) && (lastitem->id < entry->id))
                              break;
                 
                 
                           lastitem = curitem;  
                       }
                    
                 }

                 _sciDirs[dir]->insertItem(pos, entry);
 
                 
                 return 0;                     //no error
             }              
             return 1;                        //file already exists
        
        }
     }
     return -1;                          //dir not initialized 
  }
  

